options nodate nocenter nonumber ls=max ps=max;

libname crsp     '/homedir/acc/mjung/bigdata/stern/crsp/';
libname tfn      '/homedir/acc/mjung/bigdata/stern/tfn/';
libname jwz      '/homedir/acc/mjung/bigdata/stern/jwz/tar2/';
libname jwzhome  '/homedir/acc/mjung/bigdata/stern/jwz/';
libname home     '/homedir/acc/mjung/bigdata/stern/jwz2/';


***** ADD MGRNO FOR BUYSIDERS;
data temp1;
  set home.cleandata;
  if AMASKCD=888888 and EMASKCD=. then DBUYSIDE=1; 
    else DBUYSIDE=0;

proc sql; create table addmgrno as
          select temp1.*, match.MGRNO
            from temp1 left join home.match
              on temp1.FILE_NAME  =match.FILE_NAME
             and temp1.TICKER     =match.TICKER
             and temp1.CCDATE     =match.CCDATE
             and temp1.TIME       =match.TIME
             and temp1.ANALYST    =match.ANALYST
             and temp1.AFFILIATION=match.AFFILIATION;

data temp2;
  set addmgrno;
* LEGITIMATE BUY-SIDERS HAVE NON-MISSING MGRNO;
  if DBUYSIDE=1 and MGRNO ne . then LEGIT=1;
    else LEGIT=0;


***** DETERMINE CALENDAR QUARTER END BEFORE AND AFTER CC;
  if month(CCDATE) in (1,2,3)    then RDATE_t0=mdy(12,31,year(CCDATE)-1);
  if month(CCDATE) in (4,5,6)    then RDATE_t0=mdy(3,31,year(CCDATE));
  if month(CCDATE) in (7,8,9)    then RDATE_t0=mdy(6,30,year(CCDATE));
  if month(CCDATE) in (10,11,12) then RDATE_t0=mdy(9,30,year(CCDATE));

  if month(CCDATE) in (1,2,3)    then RDATE_t1=mdy(3,31,year(CCDATE));
  if month(CCDATE) in (4,5,6)    then RDATE_t1=mdy(6,30,year(CCDATE));
  if month(CCDATE) in (7,8,9)    then RDATE_t1=mdy(9,30,year(CCDATE));
  if month(CCDATE) in (10,11,12) then RDATE_t1=mdy(12,31,year(CCDATE));
  format RDATE_t0 RDATE_t1 date9.;
  DAYS_t0=CCDATE-RDATE_t0;
  DAYS_t1=RDATE_t1-CCDATE;


***** ADD TONE VARIABLES;
data temp55;
  set jwzhome.qa_parse_confqa_mjj2;
  if mdy(4,1,2002)<=CCDATE<=mdy(3,31,2009);

proc sql; create table addtone as
          select temp2.*, to.DIALOGUES, to.ANALYST_WORDS, to.ANALYST_POS, to.ANALYST_NEG, to.CEO_WORDS, to.CEO_POS, to.CEO_NEG, to.CFO_WORDS, to.CFO_POS, to.CFO_NEG
            from temp2 left join temp55 as to
              on temp2.FILE_NAME    =to.FILE_NAME
             and temp2.TICKER       =to.TICKER
             and temp2.CCDATE       =to.CCDATE
             and temp2.SET          =to.SET
             and temp2.ANALYST      =to.ANALYST
             and temp2.AFFILIATION  =to.AFFILIATION;

data addtone;
  set addtone;
* DEFINE NUMBER OF QUESTIONS VARIABLES;
  if LEGIT=1 then BS_NUMQ=DIALOGUES;
  if LEGIT=0 then SS_NUMQ=DIALOGUES;

* DEFINE NUMBER OF WORDS VARIABLES;
  if LEGIT=1 and ANALYST_WORDS>2 then BS_NUMWORDS=ANALYST_WORDS;
  if LEGIT=0 and ANALYST_WORDS>2 then SS_NUMWORDS=ANALYST_WORDS;
  if LEGIT=1 and CEO_WORDS>2 and CEO_POS<CEO_WORDS then BS_CEO_NUMWORDS=CEO_WORDS;
  if LEGIT=0 and CEO_WORDS>2 and CEO_POS<CEO_WORDS then SS_CEO_NUMWORDS=CEO_WORDS;
  if LEGIT=1 and CFO_WORDS>2 and CFO_POS<CFO_WORDS then BS_CFO_NUMWORDS=CFO_WORDS;
  if LEGIT=0 and CFO_WORDS>2 and CFO_POS<CFO_WORDS then SS_CFO_NUMWORDS=CFO_WORDS;
  BS_WORDSPERQ = BS_NUMWORDS/BS_NUMQ;
  SS_WORDSPERQ = SS_NUMWORDS/SS_NUMQ;
  BS_CEO_WORDSPERQ = BS_CEO_NUMWORDS/BS_NUMQ;
  SS_CEO_WORDSPERQ = SS_CEO_NUMWORDS/SS_NUMQ;
  BS_CFO_WORDSPERQ = BS_CFO_NUMWORDS/BS_NUMQ;
  SS_CFO_WORDSPERQ = SS_CFO_NUMWORDS/SS_NUMQ;

* DEFINE TONE VARIABLES;
  if LEGIT=1 and ANALYST_WORDS>2 then BS_TONE =(ANALYST_POS-ANALYST_NEG)/ANALYST_WORDS;
  if LEGIT=0 and ANALYST_WORDS>2 then SS_TONE =(ANALYST_POS-ANALYST_NEG)/ANALYST_WORDS;
  if LEGIT=1 and CEO_WORDS>2 and CEO_POS<CEO_WORDS then BS_CEO_TONE=(CEO_POS-CEO_NEG)/CEO_WORDS;
  if LEGIT=0 and CEO_WORDS>2 and CEO_POS<CEO_WORDS then SS_CEO_TONE=(CEO_POS-CEO_NEG)/CEO_WORDS;
  if LEGIT=1 and CFO_WORDS>2 and CFO_POS<CFO_WORDS then BS_CFO_TONE = (CFO_POS-CFO_NEG)/CFO_WORDS;
  if LEGIT=0 and CFO_WORDS>2 and CFO_POS<CFO_WORDS then SS_CFO_TONE = (CFO_POS-CFO_NEG)/CFO_WORDS;


***** ADD TFN DATA BOTH BEFORE AND AFTER CONF CALL;
data ssonly; set addtone; if LEGIT=0;
data bsonly; set addtone; if LEGIT=1;

proc sql; create table addportvalue1 as
          select bsonly.*,       pvb4.NUMFIRMS as NUMFIRMS_t0, pvb4.PORTVALUE as PORTVALUE_t0
            from bsonly left join tfn.portvalue as pvb4
              on bsonly.MGRNO   =pvb4.MGRNO
             and bsonly.RDATE_t0=pvb4.RDATE;

proc sql; create table addportvalue2 as
          select addportvalue1.*,       pvaf.NUMFIRMS as NUMFIRMS_t1, pvaf.PORTVALUE as PORTVALUE_t1
            from addportvalue1 left join tfn.portvalue as pvaf
              on addportvalue1.MGRNO   =pvaf.MGRNO
             and addportvalue1.RDATE_t1=pvaf.RDATE;

proc sql; create table addtfnb4 as
          select addportvalue2.*, tb.MGRNAME, tb.TYPECODE, tb.STKNAME, 
                                  tb.SHARES as SHARES_t0, tb.SHROUT1 as SHROUT1_t0, tb.PRC as PRC_t0, tb.VALUE as VALUE_t0, tb.PCTOFPORT as PCTOFPORT_t0
            from addportvalue2 left join tfn.pctofport2000 as tb
              on addportvalue2.TICKER  =tb.TICKER
             and addportvalue2.MGRNO   =tb.MGRNO
             and addportvalue2.RDATE_t0=tb.RDATE;

proc sql; create table addtfnafter as
          select addtfnb4.*,      ta.SHARES as SHARES_t1, ta.SHROUT1 as SHROUT1_t1, ta.PRC as PRC_t1, ta.VALUE as VALUE_t1, ta.PCTOFPORT as PCTOFPORT_t1
            from addtfnb4 left join tfn.pctofport2000 as ta
              on addtfnb4.TICKER  =ta.TICKER
             and addtfnb4.MGRNO   =ta.MGRNO
             and addtfnb4.RDATE_t1=ta.RDATE;

data addtfnafter;
  set addtfnafter;
***** BEFORE;
  if SHROUT1_t0=0 then SHROUT1_t0=.;
  PCTOWN_t0=SHARES_t0/(SHROUT1_t0*1000000);
  if SHARES_t0=. then do;
    SHARES_t0=0;
    PCTOWN_t0=0;
    VALUE_t0=0;
    PCTOFPORT_t0=0;
  end;
  LPORTVALUE_t0=log(1+PORTVALUE_t0);

***** AFTER;
  if SHROUT1_t1=0 then SHROUT1_t1=.;
  PCTOWN_t1=SHARES_t1/(SHROUT1_t1*1000000);
  if SHARES_t1=. then do;
    SHARES_t1=0;
    PCTOWN_t1=0;
    VALUE_t1=0;
    PCTOFPORT_t1=0;
  end;
  LPORTVALUE_t1=log(1+PORTVALUE_t1);

data temp3;
  set ssonly addtfnafter;


***** AGGREGATE INTO FIRM-QUARTERS;
data temp4;
  set temp3;
  if LEGIT=1 then DBUYSIDE=1;
    else DBUYSIDE=0;

* DEFINE VARIABLES FOR BUYSIDERS WHO OWN;
  if DBUYSIDE=1 and PCTOWN_t0>0 then DBUYSIDE_OWN=1;
    else DBUYSIDE_OWN=0;

* DEFINE VARIABLES FOR BUYERS;
  CPCTOWN=PCTOWN_t1-PCTOWN_t0;
  if DBUYSIDE=1 and PCTOWN_t0>0 and CPCTOWN>0 then OWNBUY=1;
    else OWNBUY=0;
  if DBUYSIDE=1 and PCTOWN_t0=0 and CPCTOWN>0 then NOTOWNBUY=1;
    else NOTOWNBUY=0;

* DEFINE VARIABLES FOR SELLERS;
  if DBUYSIDE=1 and PCTOWN_t0>0 and CPCTOWN<0 then OWNSELL=1;
    else OWNSELL=0;

* DEFINE VARIABLES FOR HOLDERS;
  if DBUYSIDE=1 and PCTOWN_t0>0 and CPCTOWN=0 then OWNHOLD=1;
    else OWNHOLD=0;
  if DBUYSIDE=1 and PCTOWN_t0=0 and CPCTOWN=0 then NOTOWNNOTBUY=1;
    else NOTOWNNOTBUY=0;

* ESTIMATE ORDER OF BS ANALYSTS IN ASKING QUESTIONS;
  if DBUYSIDE=1 then BS_ORDER=SET;

* CONSOLIDATE TO FIRM-QUARTERS;
  proc sort;
    by FILE_NAME TICKER CCDATE TIME;
  proc means noprint;
    by FILE_NAME TICKER CCDATE TIME;
    output out=mdat2 median(RDATE_t0)=RDATE_t0 
                     median(DAYS_t0)=DAYS_t0 
                     sum(PCTOWN_t0)=TOTALPCTOWN_t0 
                     median(RDATE_t1)=RDATE_t1 
                     median(DAYS_t1)=DAYS_t1 
                     sum(PCTOWN_t1)=TOTALPCTOWN_t1
                     sum(DBUYSIDE)=            NUMBUYSIDERS 
                     sum(DBUYSIDE_OWN)=        NUMBUYSIDERS_OWN
                     sum(OWNBUY) =             NUMBUYSIDERS_OWN_BUY
                     sum(OWNSELL)=             NUMBUYSIDERS_OWN_SELL
                     sum(OWNHOLD)=             NUMBUYSIDERS_OWN_HOLD
                     sum(NOTOWNBUY)=           NUMBUYSIDERS_NOTOWN_BUY
                     sum(NOTOWNNOTBUY)=        NUMBUYSIDERS_NOTOWN_NOTBUY
                     mean(PORTVALUE_t0)=       MEANPORTVALUE_t0
                     mean(LPORTVALUE_t0)=      MEANLPORTVALUE_t0
                     median(SICH)=SICH
                     median(SIC2)=SIC2
                     median(SIC3)=SIC3
                     median(QTR)=QTR
                     mean(BS_NUMQ)=MEAN_BS_NUMQ
                     mean(SS_NUMQ)=MEAN_SS_NUMQ
                     mean(BS_NUMWORDS)=MEAN_BS_NUMWORDS
                     mean(SS_NUMWORDS)=MEAN_SS_NUMWORDS
                     mean(BS_CEO_NUMWORDS)=MEAN_BS_CEO_NUMWORDS
                     mean(SS_CEO_NUMWORDS)=MEAN_SS_CEO_NUMWORDS
                     mean(BS_CFO_NUMWORDS)=MEAN_BS_CFO_NUMWORDS
                     mean(SS_CFO_NUMWORDS)=MEAN_SS_CFO_NUMWORDS
                     mean(BS_WORDSPERQ)=MEAN_BS_WORDSPERQ
                     mean(SS_WORDSPERQ)=MEAN_SS_WORDSPERQ
                     mean(BS_CEO_WORDSPERQ)=MEAN_BS_CEO_WORDSPERQ
                     mean(SS_CEO_WORDSPERQ)=MEAN_SS_CEO_WORDSPERQ
                     mean(BS_CFO_WORDSPERQ)=MEAN_BS_CFO_WORDSPERQ
                     mean(SS_CFO_WORDSPERQ)=MEAN_SS_CFO_WORDSPERQ
                     mean(BS_TONE)=MEAN_BS_TONE
                     mean(SS_TONE)=MEAN_SS_TONE
                     mean(BS_CEO_TONE)=MEAN_BS_CEO_TONE
                     mean(SS_CEO_TONE)=MEAN_SS_CEO_TONE
                     mean(BS_CFO_TONE)=MEAN_BS_CFO_TONE
                     mean(SS_CFO_TONE)=MEAN_SS_CFO_TONE
                     mean(BS_ORDER)=MEAN_BS_ORDER
                     max(SET)=MAX_SET;

data mdat2;
  set mdat2;
  drop _TYPE_;
  rename _FREQ_=NUMCALLERSJWZ2;
  RELATIVEPLACE=MEAN_BS_ORDER/MAX_SET;
* SCALE VARIABLES;
  NUMBUYSIDERSSC       =NUMBUYSIDERS       /_FREQ_;
  NUMBUYSIDERS_OWNSC   =NUMBUYSIDERS_OWN   /_FREQ_;
  NUMBUYSIDERS_NOTOWN  =NUMBUYSIDERS-NUMBUYSIDERS_OWN;
  NUMBUYSIDERS_NOTOWNSC=NUMBUYSIDERS_NOTOWN/_FREQ_;

  if TOTALPCTOWN_t0=. then TOTALPCTOWN_t0=0;
  if TOTALPCTOWN_t1=. then TOTALPCTOWN_t1=0;
  CPCTOWN=(TOTALPCTOWN_t1-TOTALPCTOWN_t0)*100;
  if CPCTOWN=. then CPCTOWN=0;
  if NUMBUYSIDERS>0 then DBUYSIDER=1;
    else DBUYSIDER=0;
  proc sort;
    by TICKER CCDATE FILE_NAME TIME;

data mdat2;
  set mdat2;
    by TICKER CCDATE FILE_NAME TIME;

  DBUYSIDER_LAG=lag(DBUYSIDER);
  if first.TICKER then DBUYSIDER_LAG=.;

  NUMBUYSIDERS_LAG=lag(NUMBUYSIDERS);
  if first.TICKER then NUMBUYSIDERS_LAG=.;

  NUMBUYSIDERS_OWNLAG=lag(NUMBUYSIDERS_OWN);
  if first.TICKER then NUMBUYSIDERS_OWNLAG=.;

  NUMBUYSIDERS_NOTOWNLAG=lag(NUMBUYSIDERS_NOTOWN);
  if first.TICKER then NUMBUYSIDERS_NOTOWNLAG=.;

  CNUMBUYSIDERS=NUMBUYSIDERS-NUMBUYSIDERS_LAG;
  CNUMBUYSIDERS_OWN=NUMBUYSIDERS_OWN-NUMBUYSIDERS_OWNLAG;
  CNUMBUYSIDERS_NOTOWN=NUMBUYSIDERS_NOTOWN-NUMBUYSIDERS_NOTOWNLAG;

  ABSCNUMBUYSIDERS=abs(CNUMBUYSIDERS);
  ABSCNUMBUYSIDERS_OWN=abs(CNUMBUYSIDERS_OWN);
  ABSCNUMBUYSIDERS_NOTOWN=abs(CNUMBUYSIDERS_NOTOWN);

data temp4;
  set mdat2;


***** ADD PERMNO;
proc sql; create table addpermno as
          select temp4.*,             dsfhdr.PERMNO
            from temp4 left join crsp.dsfhdr
              on temp4.TICKER=        dsfhdr.HTSYMBOL
             and dsfhdr.BEGDAT<=temp4.CCDATE<=dsfhdr.ENDDAT;

data hitpermno;
  set addpermno;
  if permno ne .;

data miss;
  set addpermno;
  if PERMNO=.;
  drop PERMNO;

proc sql; create table addpermno2 as
          select miss.*,             stocknames.PERMNO
            from miss left join crsp.stocknames
              on miss.TICKER=        stocknames.TICKER
             and stocknames.NAMEDT<=miss.CCDATE<=stocknames.NAMEENDDT;

data temp5;
  set hitpermno addpermno2;
  if PERMNO ne .;


***** ADD VARIABLES FROM JWZ1;
proc sql; create table addjwz1 as
          select temp5.*, regdata.NC_ANALYSTS, regdata.NC_ANALYSTSSC, regdata.COV_ANALYSTS_ABSENT, regdata.COV_ANALYSTS_ABSENTSC,
                          regdata.NUMCALLERS as NUMCALLERSJWZ1, regdata.NANBEFORE as NAN, regdata.QNAN, regdata.LMVQ, regdata.BTMQ, regdata.GVKEY
            from temp5 left join jwz.regdata
              on temp5.TICKER=regdata.TICKER
             and temp5.CCDATE=regdata.CCDATE
             and temp5.TIME  =regdata.TIME;


***** ADD ENEWS;
proc sql; create table addenews as
          select addjwz1.*, enews.ENEWS
            from addjwz1 left join ibes.enews
              on addjwz1.PERMNO=        enews.PERMNO
             and (enews.ANNDATS_ACT-1)<=addjwz1.CCDATE<=(enews.ANNDATS_ACT+1);


***** ANALYST EPS FORECAST DISPERSION;
proc sql; create table adddisp as
          select addenews.*, dispersion.DISPERSION
            from addenews left join ibes.dispersion
              on addenews.PERMNO=dispersion.PERMNO
             and dispersion.FPEDATS<addenews.CCDATE<(dispersion.FPEDATS+90);


***** ADD NII;
proc sql; create table addpio as
          select adddisp.*, nii.PIO0 as PIO_t0, nii.PIO1 as PIO_t1, nii.NII0 as NII_t0, nii.NII1 as NII_t1, nii.SHROUT1
            from adddisp left join home.nii_exclude as nii
              on adddisp.TICKER=nii.TICKER
             and nii.RDATE0<adddisp.CCDATE<nii.RDATE1;


***** ADD CRSP FIRM AGE IN MONTHS;
proc sql; create table addage as
          select addpio.*, crspdcqtr_winsor.FIRMAGEMONTH
            from addpio left join crsp.crspdcqtr_winsor as c
              on addpio.PERMNO=c.PERMNO
             and c.CDATADATE<addpio.CCDATE<(c.CDATADATE+90);

data addage;
  set addage;
  CPIO=(PIO_t1-PIO_t0)*100;
  LFIRMAGEMONTH=log(1+FIRMAGEMONTH);


***** ADD STOCK PRICE ON DATE OF CCDATE;
proc sql; create table addprc as
          select addage.*, crspd.PRC
            from addage left join crsp.crspd
              on addage.PERMNO=crspd.PERMNO
             and addage.CCDATE=crspd.DATE;
data addprc;
  set addprc;
  if PRC >= 1.00 then DISPERSIONSC=DISPERSION/PRC;
    else DISPERSIONSC=.;


*************** ADD SHORT INTEREST VARIABLE;
data sec_shortint;
  set comp.sec_shortint;
  DATAYEAR=year(DATADATE);
  DATAMONTH=month(DATADATE);
  if 2001<=DATAYEAR<=2009;
  proc sort nodupkeys;
    by GVKEY DATAYEAR DATAMONTH;

data temp5;
  set addprc;
  CCMONTH=month(CCDATE);
  CCYEAR=year(CCDATE);
  if CCMONTH ne 12;

data temp5_12;
  set addprc;
  CCMONTH=month(CCDATE);
  CCYEAR=year(CCDATE);
  if CCMONTH=12;

proc sql; create table addshortint1 as
          select temp5.*, sec_shortint.DATADATE as SHORTINT_AFTERDATE, sec_shortint.SHORTINTADJ as SHORTINTADJ_AFTER
            from temp5 left join   sec_shortint
              on temp5.GVKEY      =sec_shortint.GVKEY
             and temp5.CCYEAR     =sec_shortint.DATAYEAR
             and (temp5.CCMONTH+1)=sec_shortint.DATAMONTH;

proc sql; create table addshortint2 as
          select temp5_12.*, sec_shortint.DATADATE as SHORTINT_AFTERDATE, sec_shortint.SHORTINTADJ as SHORTINTADJ_AFTER
            from temp5_12 left join     sec_shortint
              on temp5_12.GVKEY        =sec_shortint.GVKEY
             and (temp5_12.CCYEAR+1)   =sec_shortint.DATAYEAR
             and sec_shortint.DATAMONTH=1;

data addshortint;
  set addshortint1 addshortint2;

data temp6;
  set addshortint;
  if CCMONTH ne 1;

data temp6_1;
  set addshortint;
  if CCMONTH=1;

proc sql; create table addshortint3 as
          select temp6.*, sec_shortint.DATADATE as SHORTINT_BEFOREDATE, sec_shortint.SHORTINTADJ as SHORTINTADJ_BEFORE
            from temp6 left join   sec_shortint
              on temp6.GVKEY      =sec_shortint.GVKEY
             and temp6.CCYEAR     =sec_shortint.DATAYEAR
             and (temp6.CCMONTH-1)=sec_shortint.DATAMONTH;

proc sql; create table addshortint4 as
          select temp6_1.*, sec_shortint.DATADATE as SHORTINT_BEFOREDATE, sec_shortint.SHORTINTADJ as SHORTINTADJ_BEFORE
            from temp6_1 left join   sec_shortint
              on temp6_1.GVKEY      =sec_shortint.GVKEY
             and (temp6_1.CCYEAR-1) =sec_shortint.DATAYEAR
             and sec_shortint.DATAMONTH=12;

data addshortint5;
  set addshortint3 addshortint4;
  if SHORTINTADJ_BEFORE=. then SHORTINTADJ_BEFORE=0;
  if SHORTINTADJ_AFTER=.  then SHORTINTADJ_AFTER=0;
  PCTSHORT_BEFORE=SHORTINTADJ_BEFORE/(SHROUT90*1000);
  PCTSHORT_AFTER =SHORTINTADJ_AFTER/(SHROUT90*1000);
  proc sort;
    by TICKER CCDATE TIME;

data addshortint6;
  set addshortint5;
    by TICKER CCDATE;
  CSHORTINT=(PCTSHORT_AFTER-PCTSHORT_BEFORE)*100;
  ABSCSHORTINT=abs(CSHORTINT);


***** ADD INTRADAY;
data addintraday;
  set addshortint6;
  if TIME in ('1:00pm et','1:20pm et','1:30pm et','10:00am et','10:15am et','10:30am et','10:45am et','11:00am et','11:01am et','11:15am et','11:30am et','11:45am et',
              '12:00pm et','12:15pm et','12:30pm et','2:00pm et','2:15pm et','2:30pm et','3:00pm et','3:30pm et',
              '9:30am et','9:45am et',
              '02:30pm  gmt','02:45pm  gmt','03:00pm  gmt','03:15pm  gmt','03:30pm  gmt','03:45pm  gmt','04:00pm  gmt',
              '04:15pm  gmt','04:30pm  gmt','05:00pm  gmt','05:30pm  gmt','06:00pm  gmt','06:30pm  gmt','07:00pm  gmt','07:15pm  gmt','07:30pm  gmt','08:00pm  gmt','08:15pm  gmt',
              '08:30pm  gmt','08:35pm  gmt','11:00am  gmt','11:30am  gmt','12:00pm  gmt','12:15pm  gmt','12:25pm  gmt','12:30pm  gmt','12:45pm  gmt')
              then INTRADAY=1; else INTRADAY=0;


***** ADD FAMA FRENCH INDUSTRIES;
data famafrench10;
  set addintraday;

if 0100<=SICH<=0999 or
   2000<=SICH<=2399 or
   2700<=SICH<=2749 or
   2770<=SICH<=2799 or
   3100<=SICH<=3199 or
   3940<=SICH<=3989 then FF1=1; else FF1=0;

if 2500<=SICH<=2519 or
   2590<=SICH<=2599 or
   3630<=SICH<=3659 or
   3710<=SICH<=3711 or
   3714<=SICH<=3714 or
   3716<=SICH<=3716 or
   3750<=SICH<=3751 or
   3792<=SICH<=3792 or
   3900<=SICH<=3939 or
   3990<=SICH<=3999 then FF2=1; else FF2=0;

if 2520<=SICH<=2589 or
   2600<=SICH<=2699 or
   2750<=SICH<=2769 or
   2800<=SICH<=2829 or
   2840<=SICH<=2899 or
   3000<=SICH<=3099 or
   3200<=SICH<=3569 or
   3580<=SICH<=3621 or
   3623<=SICH<=3629 or
   3700<=SICH<=3709 or
   3712<=SICH<=3713 or
   3715<=SICH<=3715 or
   3717<=SICH<=3749 or
   3752<=SICH<=3791 or
   3793<=SICH<=3799 or
   3860<=SICH<=3899 then FF3=1; else FF3=0;

if 1200<=SICH<=1399 or
   2900<=SICH<=2999 then FF4=1; else FF4=0;

if 3570<=SICH<=3579 or
   3622<=SICH<=3622 or
   3660<=SICH<=3692 or
   3694<=SICH<=3699 or
   3810<=SICH<=3839 or
   7370<=SICH<=7372 or
   7373<=SICH<=7373 or
   7374<=SICH<=7374 or
   7375<=SICH<=7375 or
   7376<=SICH<=7376 or
   7377<=SICH<=7377 or
   7378<=SICH<=7378 or
   7379<=SICH<=7379 or
   7391<=SICH<=7391 or
   8730<=SICH<=8734 then FF5=1; else FF5=0;

if 4800<=SICH<=4899 then FF6=1; else FF6=0;

if 5000<=SICH<=5999 or
   7200<=SICH<=7299 or
   7600<=SICH<=7699 then FF7=1; else FF7=0;

if 2830<=SICH<=2839 or
   3693<=SICH<=3693 or
   3840<=SICH<=3859 or
   8000<=SICH<=8099 then FF8=1; else FF8=0;

if 4900<=SICH<=4949 then FF9=1; else FF9=0;

if FF1=0 and FF2=0 and FF3=0 and FF4=0 and FF5=0 and FF6=0 and FF7=0 and FF8=0 and FF9=0 then FF10=1; else FF10=0;


***** ADD LCONCURRENTCALLS;
data temp33;
  set famafrench10;
  if FF1=1 then FFIND=1;
  if FF2=1 then FFIND=2;
  if FF3=1 then FFIND=3;
  if FF4=1 then FFIND=4;
  if FF5=1 then FFIND=5;
  if FF6=1 then FFIND=6;
  if FF7=1 then FFIND=7;
  if FF8=1 then FFIND=8;
  if FF9=1 then FFIND=9;
  if FF10=1 then FFIND=10;
  proc sort;
    by FFIND CCDATE TIME;
  proc means noprint;
    by FFIND CCDATE TIME;
    output out=mdatconc mean(FFIND)=meanFFIND;

data mdatconc;
  set mdatconc;
  drop _TYPE_ meanFFIND;
  CONCURRENTCALLS=_FREQ_;
  LCONCURRENTCALLS=log(CONCURRENTCALLS);

proc sql; create table addconc as
          select temp33.*, mdatconc.LCONCURRENTCALLS
            from temp33 left join mdatconc
              on temp33.FFIND =mdatconc.FFIND
             and temp33.CCDATE=mdatconc.CCDATE
             and temp33.TIME  =mdatconc.TIME;


endsas;
